import static org.junit.Assert.*;
import org.junit.Test;

/* A value is an integer, string, or error value
 * 
 * A function is either
 *    - a value
 *    - an '=' character followed by an expression
 *    
 * An expression is either
 *    - a value
 *    - a cell reference
 *    - two expressions separated by an operator
 *    
 * An operator is one of '+', '-', '*', '/', '%', '^'      
 */


public class FunctionTest {

	@Test
	public void testNull() {
		Spreadsheet s = new Spreadsheet(); 
		Cell c        = s.getCell(0,0);
		assertEquals(c.getFunction(), null);
	}

	@Test
	public void testEmpty() {
		Spreadsheet s = new Spreadsheet(); 
		Cell c        = s.getCell(0,0);
		c.setFunction("");
		assertEquals(c.getValue(), null);
	}

	@Test
	public void testZero() {
		Spreadsheet s = new Spreadsheet(); 
		Cell c        = s.getCell(0,0);
		c.setFunction("0");
		assertEquals(c.getValue(), new IntValue(0));
	}

	@Test
	public void testTwo() {
		Spreadsheet s = new Spreadsheet(); 
		Cell c        = s.getCell(0,0);
		c.setFunction("2");
		assertEquals(c.getValue(), new IntValue(2));
	}

	@Test
	public void test10() {
		Spreadsheet s = new Spreadsheet(); 
		Cell c        = s.getCell(0,0);
		c.setFunction("10");
		assertEquals(c.getValue(), new IntValue(10));
	}


	@Test
	public void testAdd() {
		Spreadsheet s = new Spreadsheet(); 
		Cell c        = s.getCell(0,0);
		c.setFunction("=1+2");
		assertEquals(c.getValue(), new IntValue(3));
	}

	@Test
	public void testAdd3() {
		Spreadsheet s = new Spreadsheet(); 
		Cell c        = s.getCell(0,0);
		c.setFunction("=1+1+1");
		assertEquals(c.getValue(), new IntValue(3));
	}

	// Uninitialized cells are 0
	@Test
	public void testUnInitRef() {
		Spreadsheet s = new Spreadsheet(); 
		Cell c        = s.getCell(0,0);
		c.setFunction("=A1");
		assertEquals(c.getValue(), new IntValue(0));
	}

	@Test
	public void testSelfRef() {
		Spreadsheet s = new Spreadsheet(); 
		Cell c        = s.getCell(0,0);
		c.setFunction("=A0");
		assertEquals(c.getValue(), new ErrorValue("#REF!"));
	}

	@Test
	public void testCircleRef() {
		Spreadsheet s = new Spreadsheet(); 
		Cell a0        = s.getCell(0,0);
		a0.setFunction("=A1");
		assertEquals(a0.getValue(), new IntValue(0));
		Cell a1        = s.getCell(0,1);
		a1.setFunction("=A0");
		assertEquals(a0.getValue(), new ErrorValue("#REF!"));
		assertEquals(a1.getValue(), new ErrorValue("#REF!"));
	}


	@Test
	public void testDivZero() {
		Spreadsheet s = new Spreadsheet(); 
		Cell c        = s.getCell(0,0);
		c.setFunction("=3/0");
		assertEquals(c.getValue(), new ErrorValue("#DIV/0!"));
	}


	public void badParse(String string) {
		Spreadsheet s = new Spreadsheet(); 
		Cell c        = s.getCell(0,0);
		c.setFunction(string);
		assertEquals(c.getValue(), new ErrorValue("#ERROR!"));
	}

	@Test public void testBadParseRef() {
		badParse("A0");
	}

	@Test public void testBadParseGarbage() {
		badParse("1A");
	}

	@Test public void testBadParseOpen() {
		badParse("(1");
	}
	@Test public void testBadParseClose() {
		badParse("1)");
	}

}
